var should=require("should");
var assert=require("assert");
var path=require("path");

var ResponseObject=require(path.join(__dirname, "../TDPAHCMSResponseObject.js"));

describe("Response Object tests", function()
{
	it("should return an object (with valid defaults and no options)", function()
	{
		new ResponseObject(function(responseObject)
		{		
			responseObject.getObject(function(ro)
			{	
				ro.should.be.an.object;	
				ro.should.not.eql(null);
				ro.status.should.eql(200);
				(ro.entity===null).should.be.true;
				(ro.err===null).should.be.true;
				ro.ttl.should.eql(0);
				ro.stime.should.eql(0);
			});
		});
	});

	it("should return an object (with valid defaults)", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.getObject(function(ro)
			{
				ro.should.be.an.object;	
				ro.should.not.eql(null);	
			});
		});
	});

	it("should throw an error with an invalid options argument", function()
	{
		var options="badger";

		(function()
		{
			new ResponseObject(options, function(responseObject)
			{			
			});

		}).should.throw();
	});

	it("should be the same as testObject when testObject is valid and supplied in the constructor", function()
	{
		var testObject=
		{
			status:200,
			entity:"Some content or other...",
			err:"Some error or other...",
			ttl:3600,
			stime:123
		};

		new ResponseObject(testObject, function(responseObject)
		{			
			responseObject.getObject(function(ro)
			{
				assert.deepEqual(ro, testObject);
			});
		});
	});

	it("should throw an error if an invalid property value", function()
	{
		var testObject=
		{
			status:101,
			entity:"Some content or other...",
			err:"Some error or other...",
			ttl:3600,
			stime:123
		};

		(function()
		{
			new ResponseObject(responseObject, function(responseObject)
			{			
				responseObject.getObject(function(ro)
				{
					assert.deepEqual(ro, responseObject);
				});
			});
		}).should.throw();
	});

	it("should throw if an invalid property is specified", function()
	{
		var testObject=
		{
			thisIsNotTheStatus:100,
			entity:"Some content or other",
			err:"Some error or other",
			ttl:3600,
			stime:10
		};

		(function()
		{
			new ResponseObject(responseObject, function(responseObject)
			{			
				responseObject.getObject(function(ro)
				{
					assert.notDeepEqual(ro, responseObject);
				});
			});
		}).should.throw();
	});

	it("should be the same as testObject when testObject is valid", function()
	{
		var testObject=
		{
			status:404,
			entity:"Some content or other",
			err:"Some error or other",
			ttl:3600,
			stime:10
		}

		new ResponseObject(function(responseObject)
		{
			responseObject.setObject(testObject, function(ro)
			{
				responseObject.getObject(function(ro)
				{
					assert.deepEqual(ro, testObject);
				});
			});
		});
	});

	it("should throw when testObject is invalid", function()
	{
		var testObject=
		{
			status:100,
			entity:"Some content or other",
			err:"Some error or other",
			ttl:3600,
			stime:10
		};

		
		new ResponseObject(function(responseObject)
		{
			(function()
			{
				responseObject.setObject(testObject, function(ro)
				{
				});
			}).should.throw();
		});
	});


	// it("should return false when testObject is not an object (null)", function()
	it("should throw an error when testObject is not an object (null)", function()
	{
		var testObject=null;

		new ResponseObject(function(responseObject)
		{
			(function()
			{
			
				responseObject.setObject(testObject, function(ro)
				{			
				});
			}).should.throw();
		});
	});



	// it("should return false when testObject is not an object (string)", function()
	it("should throw an error when testObject is not an object (string)", function()
	{
		var testObject="aasasas";

		new ResponseObject(function(responseObject)
		{
			(function()
			{
				responseObject.setObject(testObject, function(ro)
				{			
				});
			}).should.throw();
		});
	});

	it("should return an object when testObject is empty", function()
	{
		var testObject=
		{
		}

		new ResponseObject(function(responseObject)
		{
			responseObject.setObject(testObject, function(ro)
			{
				ro.should.be.an.object;
			});
		});
	});


	it("should throw if set() is used to try to set an object", function()
	{
		var testObject=
		{
			status:500,
			entity:"Some content or other",
			err:"Some error or other",
			ttl:3600,
			stime:10
		};

		
		new ResponseObject(function(responseObject)
		{
			(function()
			{
				responseObject.set(testObject, function(ro)
				{			
				});
			}).should.throw();
		});
	});

	
// STATUS tests
	it("should return true for a valid status integer", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.set("status", 404, function(ro)
			{
				// ro.should.eql(true);
				ro.should.be.an.object;
			});
		});
	});

	it("should return false for an invalid status integer", function()
	{
		new ResponseObject(function(responseObject)
		{		
			responseObject.set("status", 999, function(ro)
			{
				ro.should.eql(false);
			});
		});
	});

	it("should return false for an non-integer (string) status", function()
	{
		new ResponseObject(function(responseObject)
		{		
			responseObject.set("status", "abc", function(ro)
			{
				ro.should.eql(false);
			});
		});
	});

	it("should return false for an non-integer (object) status", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.set("status", {a:"b",c:2000}, function(ro)
			{
				ro.should.eql(false);
			});
		});
	});

	it("should return false for an non-integer (null) status", function()
	{
		new ResponseObject(function(responseObject)
		{
			var ro=responseObject.set("status", null, function(ro)
			{
				ro.should.eql(false);
			});
		});
	});


// entity tests
	it("should return true for a string entity", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.set("entity", 100, function(ro)
			{
				ro.should.be.an.object;
			});
		});
	});

	it("should return true for an non-integer entity", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.set("entity", "abc", function(ro)
			{
				ro.should.be.an.object;
			});
		});
	});

	it("should return true for an non-integer (object) entity", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.set("entity", {a:"b",c:2000}, function(ro)
			{
				ro.should.be.an.object;
			});
		});
	});

	it("should return true for an non-integer (null) entity", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.set("entity", null, function(ro)
			{
				ro.should.be.an.object;
			});
		});
	});

	it("should return true for an empty string entity", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.set("entity", "", function(ro)
			{
				ro.should.be.an.object;
			});
		});
	});

	it("should return true for a boolean entity", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.set("entity", true, function(ro)
			{
				ro.should.be.an.object;
			});
		});
	});


// err tests
	it("should return true for a string err", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.set("err", 100, function(ro)
			{
				ro.should.be.an.object;
			});
		});
	});

	it("should return true for an non-integer err", function()
	{
		new ResponseObject(function(responseObject)
		{
			var ro=responseObject.set("err", "abc", function(ro)
			{
				ro.should.be.an.object;
			});
		});
	});

	it("should return true for an non-integer (object) err", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.set("err", {a:"b",c:2000}, function(ro)
			{
				ro.should.be.an.object;
			});
		});
	});

	it("should return true for an non-integer (null) err", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.set("err", null, function(ro)
			{
				ro.should.be.an.object;
			});
		});
	});

	it("should return true for an empty string err", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.set("err", "", function(ro)
			{
				ro.should.be.an.object;
			});
		});
	});

	it("should return true for a boolean err", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.set("err", true, function(ro)
			{
				ro.should.be.an.object;
			});
		});
	});


// TTL tests
	it("should return true for an integer ttl", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.set("ttl", 100, function(ro)
			{
				ro.should.be.an.object;
			});
		});
	});

	it("should return false for an non-integer ttl", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.set("ttl", "abc", function(ro)
			{
				ro.should.eql(false);
			});
		});
	});

	it("should return false for an non-integer (object) ttl", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.set("ttl", {a:"b",c:2000}, function(ro)
			{
				ro.should.eql(false);
			});
		});
	});

	it("should return false for an non-integer (null) ttl", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.set("ttl", null, function(ro)
			{
				ro.should.eql(false);
			});
		});
	});


// STIME tests
	it("should return true for an integer stime", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.set("stime", 100, function(ro)
			{
				ro.should.be.an.object;
			});
		});
	});

	it("should return false for an non-integer (string) stime", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.set("stime", "abc", function(ro)
			{
				ro.should.eql(false);
			});
		});
	});

	it("should return false for an non-integer (object) stime", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.set("stime", {a:"b",c:2000}, function(ro)
			{
				ro.should.eql(false);
			});
		});
	});

	it("should return false for an non-integer (null) stime", function()
	{
		new ResponseObject(function(responseObject)
		{
			responseObject.set("stime", null, function(ro)
			{
				ro.should.eql(false);
			});
		});
	});
});